# frozen_string_literal: true

module Notifiable
  class Notification < ActiveRecord::Base
    self.table_name_prefix = 'notifiable_'

    belongs_to :app, class_name: 'Notifiable::App'
    validates :app, presence: true

    serialize :parameters

    has_many :notification_statuses, class_name: 'Notifiable::NotificationStatus', dependent: :destroy
    
    after_initialize :set_default_status
    
    ERROR_STATUS = -1
    WAITING_STATUS = 0
    SENDING_STATUS = 1
    FINISHED_STATUS = 2

    def batch
      update(status: SENDING_STATUS)
      yield(self)
      update_attributes({status: FINISHED_STATUS, last_error_message: nil})
    rescue => e
      update_attributes({status: ERROR_STATUS, last_error_message: e.message})
      raise e
    ensure
      close
    end

    def add_device_token(d)
      provider = d.provider.to_sym

      unless notifiers[provider]
        clazz = Notifiable.notifier_class(self, d)
        raise "Notifier #{provider} not configured" unless clazz
        notifier = clazz.new(self)
        app.configure(provider, notifier)
        @notifiers[provider] = notifier
      end

      notifiers[provider].send_notification(d)
    end

    def send_params
      @send_params ||= (parameters || {}).merge(n_id: id)
    end
    
    def delivered!(device_token)
      if notification_statuses.exists?(device_token: device_token)
        s = notification_statuses.find_by(device_token: device_token)
        return if s.delivered?
        s.delivered!
      end
      increment!(:delivered_count)
    end
    
    def opened!(device_token)
      if notification_statuses.exists?(device_token: device_token)
        s = notification_statuses.find_by(device_token: device_token)
        return if s.opened?
        s.opened!
      end
      increment!(:opened_count)
    end

    private

    def set_default_status
      self.status ||= WAITING_STATUS
    end

    def notifiers
      @notifiers ||= {}
    end

    def close
      notifiers.each_value(&:close)
      @notifiers = nil
    end
  end
end
